<?php
// $Id: luceneapi_node.index.inc,v 1.1.2.16 2009/08/10 01:40:13 cpliakas Exp $

/**
 * @file
 * Index maintenance functions for Search Lucene Content.
 *
 * @ingroup luceneapi
 */

/**
 * Shutdown function for Lucene index updates.
 *
 * @return
 *   NULL
 * @throws LuceneAPI_Exception
 * @see luceneapi_node_update_index()
 */
function _luceneapi_node_update_index_shutdown() {
  global $_luceneapi_node;
  if (is_array($_luceneapi_node) && $_luceneapi_node['last_change'] && $_luceneapi_node['last_nid']) {
    variable_set('luceneapi_node:last_change', $_luceneapi_node['last_change']);
    variable_set('luceneapi_node:last_nid', $_luceneapi_node['last_nid']);
  }
}

/**
 * Called by hook_update_index() implementation.
 */
function _luceneapi_node_update_index() {
  $log_info = luceneapi_log_level_compare(WATCHDOG_INFO);
  if ($log_info) {
    watchdog('luceneapi_node', 'Index update started.', array(), WATCHDOG_INFO);
  }

  // variables that must persist to the shutdown function
  global $_luceneapi_node;
  $_luceneapi_node = array(
    'last_change' => NULL,
    'last_nid' => variable_get('luceneapi_node:last_nid', 0),
  );

  // registers shutdown function to store the last node that was indexed
  register_shutdown_function('_luceneapi_node_update_index_shutdown');

  // gets last cron change, limit on how many nodes can be indexed
  $last = variable_get('luceneapi_node:last_change', 0);
  $limit = luceneapi_variable_get('luceneapi_node', 'update_limit');

  // SQL that gets nodes that have changed since the last Lucene index update
  $sql = 'SELECT GREATEST(IF(c.last_comment_timestamp IS NULL, 0, c.last_comment_timestamp), n.changed) as last_change, n.nid'
       .' FROM {node} n'
       .' LEFT JOIN {node_comment_statistics} c ON n.nid = c.nid'
       .' WHERE n.status = 1'
       .'   AND ('
       .'     (GREATEST(n.changed, c.last_comment_timestamp) = %d AND n.nid > %d) OR'
       .'     (n.changed > %d OR c.last_comment_timestamp > %d)'
       .'   )'
       .' ORDER BY GREATEST(n.changed, c.last_comment_timestamp) ASC, n.nid ASC';

  // adds statement to ignore excluded content types
  $sql = luceneapi_node_type_condition_add($sql);

  // array of parameters passed to db_query_range()
  $params = array($last, $_luceneapi_node['last_nid'], $last, $last, $last);

  try {

    // bails if we cannot open the index, $errstr is returned translated
    if (!$index = luceneapi_index_open('luceneapi_node', $errstr)) {
      throw new LuceneAPI_Exception($errstr);
    }

    // flags whether at least one node was changed
    $changed = FALSE;

    // iterates over changed nodes, updates index or adds document to index
    $result = db_query_range($sql, $params, 0, $limit);
    while ($node = db_fetch_object($result)) {
      if (!$changed) {
        register_shutdown_function(
          'cache_clear_all', 'luceneapi_node', LUCENEAPI_RESULTS_CACHE_TABLE, TRUE
        );
        $changed = TRUE;
      }
      $_luceneapi_node = array(
        'last_change' => $node->last_change,
        'last_nid' => $node->nid,
      );
      _luceneapi_node_document_add($index, $node->nid);
    }

    // optimizes if option is set
    if (luceneapi_variable_get('luceneapi_node', 'optimize_on_update')) {
      luceneapi_index_optimize($index, TRUE);
      if ($log_info) {
        watchdog('luceneapi_node', 'Index optimized.', array(), WATCHDOG_INFO);
      }
    }
    if ($log_info) {
      watchdog('luceneapi_node', 'Index update completed.', array(), WATCHDOG_INFO);
    }
  }
  catch (Exception $e) {
    luceneapi_throw_error($e, NULL, 'luceneapi_node');
  }
}

/**
 * Adds or replaces a document in the Lucene index with the node id $nid.
 *
 * @param $index
 *   A Zend_Search_Lucene_Interface object.
 * @param $nid
 *   An integer containing a node ID of the node being indexed.
 * @return
 *   NULL
 * @throws LuceneAPI_Exception
 * @throws Zend_Search_Exception
 */
function _luceneapi_node_document_add(Zend_Search_Lucene_Interface $index, $nid) {

  // builds node object, deletes from index if node cannot be loaded
  if ($node = node_load($nid)) {
    $node->build_mode = NODE_BUILD_SEARCH_INDEX;
    $node = node_build_content($node, FALSE, FALSE);
    $node->body = drupal_render($node->content);
    node_invoke_nodeapi($node, 'alter');
  }
  else {
    $node = new stdClass();
    $node->nid = $nid;
    luceneapi_document_delete($index, $node->nid, 'nid', $node, TRUE);
    return;
  }

  // 'contents' includes title, body, and "index only" text
  $text = luceneapi_strip_tags(join(' ', array(
    check_plain($node->title),
    $node->body,
    join(' ', node_invoke_nodeapi($node, 'update index'))
  )));

  // strips tags from title, maintains work boundaries
  $title = luceneapi_strip_tags($node->title);

  // initializes document, adds body text, title to document
  $doc = luceneapi_document_get();
  luceneapi_field_add($doc, 'unstored', 'contents', $text, TRUE);
  luceneapi_field_add($doc, 'unstored', 'title', $title, TRUE);
  luceneapi_field_add($doc, 'keyword', 'title_sort', drupal_strtolower($title), TRUE);

  // created keyword fields matching node properties
  $keywords = array(
    'changed', 'created', 'moderate', 'name', 'nid', 'promote', 'sticky',
    'tnid', 'type', 'uid',
  );
  foreach ($keywords as $field) {
    // $property = ('title_sort' != $field) ? $field : 'title';
    $text = drupal_strtolower((string)$node->$field);
    luceneapi_field_add($doc, 'keyword', $field, $text, TRUE);
  }

  // gets name of content type to index
  $type_name = node_get_types('name', $node);
  luceneapi_field_add($doc, 'unstored', 'type_name', $type_name, TRUE);

  // adds comment count to index
  $count = isset($node->comment_count) ? $node->comment_count : 0;
  luceneapi_field_add($doc, 'keyword', 'comment_count', $count, TRUE);

  // formates the translate boolean, language field
  luceneapi_field_add($doc, 'keyword', 'translate', (int)$node->translate, TRUE);
  $language = (!empty($node->language)) ? $node->language : '';
  luceneapi_field_add($doc, 'keyword', 'language', $language, TRUE);

  // indexes taxonomy terms
  if (module_exists('taxonomy')) {
    $tids = array();
    foreach ($node->taxonomy as $taxonomy) {
      $tids[] = $taxonomy->tid;
    }
    $text = join(' ', $tids);
    luceneapi_field_add($doc, 'text', 'category', $text, TRUE);
  }

  // adds nodeaccess grants as nodeaccess_ fields
  if (count(module_implements('node_grants'))) {

    // gets node access grants
    $sql = 'SELECT realm, gid'
         .' FROM {node_access}'
         .' WHERE nid = %d AND grant_view = 1';
    $result = db_query($sql, $node->nid);

    //builds array of fields to values
    $grants = array();
    while ($grant = db_fetch_object($result)) {
      $field = sprintf('nodeaccess_%s', $grant->realm);
      $grants[$field][] = $grant->gid;
    }

    // appends grants to the corresponding nodeaccess field
    foreach ($grants as $field => $values) {
      luceneapi_field_add($doc, 'text', $field, join(' ', $values), TRUE);
    }
  }

  // deletes document if it exists, adds new document to index
  luceneapi_document_delete($index, $node->nid, 'nid', $node, TRUE);
  luceneapi_document_add($index, $doc, $node, TRUE);
}
