<?php
// $Id: luceneapi.admin.inc,v 1.24.2.29 2009/08/14 01:39:16 cpliakas Exp $

/**
 * @file
 * Administrative settings callbacks for the Search Lucene API module.
 *
 * @ingroup luceneapi
 */

/**
 * Administrative settings for Search Lucene API.
 *
 * @return
 *   An array of form elements.
 * @ingroup forms
 */
function luceneapi_admin_settings() {
  $form = array();

  // index settings fieldset
  $form['global'] = array(
    '#type' => 'fieldset',
    '#title' => t('Global settings'),
    '#collapsible' => TRUE,
    '#description' => t(
      'Global search settings for all Search Lucene API modules'
    ),
  );

  // builds array of Search Lucene API modules
  $files = module_rebuild_cache();
  $hijack_options = array('0' => 'No');
  foreach (luceneapi_module_list() as $module) {
    $hijack_options[$module] = $files[$module]->info['name'];
  }

  // determines which Search Lucene API module will hijack the search box
  $form['global']['luceneapi:default_search'] = array(
    '#type' => 'radios',
    '#title' => t('Hijack search box'),
    '#default_value' => luceneapi_setting_get('default_search'),
    '#options' => $hijack_options,
    '#description' => t(
      'Determines which module will take over the default search box and search block.'
    ),
  );

  // only displays option if there are more than one analyzer
  $analyzers = module_invoke_all('luceneapi_analyzer');
  if (count($analyzers) > 1) {
    $description = 'The analyzer is responsible for parsing the text and'
                 .' preparing it for indexing.  Changing this setting <b>requires'
                 .' that all indexes be rebuilt</b>.';

    // determines which Search Lucene API module will hijack the search box
    $form['global']['luceneapi:analyzer'] = array(
      '#type' => 'radios',
      '#title' => t('Text analyzer'),
      '#default_value' => luceneapi_setting_get('analyzer'),
      '#options' => $analyzers,
      '#description' => t($description),
    );
  }

  $description = 'A list of words that are not indexed.  Words are separated by'
               .' spaces and new lines.  See the %directory directory for lists'
               .' of stopwords in common languages.';

  // determines which Search Lucene API module will hijack the search box
  $form['global']['luceneapi:stopwords'] = array(
    '#type' => 'textarea',
    '#title' => t('Stop words'),
    '#default_value' => luceneapi_setting_get('stopwords'),
    '#description' => t($description, array(
      '%directory' => drupal_get_path('module', 'luceneapi') .'/conf'
    )),
  );

  // error handling field set
  $form['error'] = array(
    '#type' => 'fieldset',
    '#title' => t('Errors and logging'),
    '#collapsible' => TRUE,
  );

  $form['error']['luceneapi:min_log_level'] = array(
    '#type' => 'radios',
    '#title' => t('Logging level'),
    '#default_value' => luceneapi_setting_get('min_log_level'),
    '#options' => array(
      WATCHDOG_ERROR => 'Error',
      WATCHDOG_INFO => 'Info',
      WATCHDOG_DEBUG => 'Debug',
    ),
    '#description' => t('The minimum severity of watchdog messages.'),
  );

  // description of error message
  $description = 'Error displayed to users who do not have %perm access. If'
               .' empty, no error will be displayed.';

  // the default error message when general mode is enabled
  $form['error']['luceneapi:error_message'] = array(
    '#type' => 'textfield',
    '#title' => t('Generic error message'),
    '#default_value' => luceneapi_setting_get('error_message'),
    '#maxlength' => 255,
    '#description' => t($description, array('%perm' => 'view luceneapi errors')),
  );


  // permissions fieldset
  $form['permissions'] = array(
    '#type' => 'fieldset',
    '#title' => t('Filesystem permissions'),
    '#collapsible' => TRUE,
    '#description' => t(
      'Sets permissions of index files for systems and installtions that support it.'
    ),
  );

  $options = array(
    0 => t('None'),
    4 => t('Read only'),
    6 => t('Read and write'),
  );

  // group permissions
  $form['permissions']['luceneapi:permissions_group'] = array(
    '#type' => 'select',
    '#title' => t('Group'),
    '#default_value' => luceneapi_setting_get('permissions_group'),
    '#options' => $options,
    '#description' => t('Group permission for the index files.'),
  );

  $description = 'Permissions for all users on the system.  Granting too many'
               .' privileges to others may expose your installation to security'
               .' risks.';

  // other permissions
  $form['permissions']['luceneapi:permissions_other'] = array(
    '#type' => 'select',
    '#title' => t('Others'),
    '#default_value' => luceneapi_setting_get('permissions_other'),
    '#options' => $options,
    '#description' => t($description),
  );

  return system_settings_form($form);
}

/**
 * Adds common submit and validate handlers for Search Lucene API settings
 * forms, passes module as a hidden element.
 */
function luceneapi_system_settings_form($form, $module) {
  $form['module'] = array(
    '#type' => 'hidden',
    '#value' => $module,
  );
  $form['#validate'][] = 'luceneapi_admin_settings_form_validate';
  $form['#submit'][] = 'luceneapi_admin_settings_form_submit';
  return system_settings_form($form);
}

/**
 * Basic administrative settings.
 *
 * @param &$form_state
 *   A keyed array containing the current state of the form.
 * @param $module
 *   A string containing the Search Lucene API module.
 * @return
 *   A FAPI array passed through system_settings_form().
 */
function luceneapi_admin_settings_general_form(&$form_state, $module) {
  $form = array();

  // index settings fieldset
  $form['index'] = array(
    '#type' => 'fieldset',
    '#title' => t('Index settings'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );

  $description = 'A filesystem path where the Lucene index is stored. In most'
               .' cases, the default setting won&apos;t need to be changed.';

  // path to index directory
  $form['index']["{$module}:index_path"] = array(
    '#type' => 'textfield',
    '#title' => t('Index path'),
    '#default_value' => luceneapi_variable_get($module, 'index_path'),
    '#maxlength' => 255,
    '#description' => t($description),
    '#after_build' => array('luceneapi_after_build'),
  );

  // index update throttle
  $options = drupal_map_assoc(array(-1, 10, 20, 50, 100, 200, 500));
  $options[-1] = 'Inherit from core search';
  $form['index']["{$module}:update_limit"] = array(
    '#type' => 'select',
    '#title' => t('Items to index per update'),
    '#default_value' => luceneapi_variable_get($module, 'update_limit', TRUE),
    '#options' => $options,
    '#description' => t('The maximum number of items that will be indexed per update.'),
  );

  // option to optimize the index
  $form['index']['reindex'] = array(
    '#type' => 'submit',
    '#value' => t('Re-index'),
  );

  // option to optimize the index
  $form['index']['wipe'] = array(
    '#type' => 'submit',
    '#value' => t('Wipe index'),
  );

  // index settings fieldset
  $form['search'] = array(
    '#type' => 'fieldset',
    '#title' => t('Search page settings'),
    '#collapsible' => TRUE,
  );

  // default results per page for all modules that implement SLAPI
  $form['search']["{$module}:results_per_page"] = array(
    '#type' => 'select',
    '#title' => t('Results per page'),
    '#default_value' => luceneapi_variable_get($module, 'results_per_page'),
    '#options' => drupal_map_assoc(array(5, 10, 20, 30, 50, 100)),
    '#description' => t('Number of items that will be shown per search results page.'),
  );

  // default operator for user generated queries
  $form['search']["{$module}:default_operator"] = array(
    '#type' => 'select',
    '#title' => t('Default operator'),
    '#default_value' => luceneapi_variable_get($module, 'default_operator'),
    '#options' => array(
      Zend_Search_Lucene_Search_QueryParser::B_AND => 'AND',
      Zend_Search_Lucene_Search_QueryParser::B_OR  => 'OR',
    ),
    '#description' => t(
      'Default operator for queries submitted through the search form.'
    ),
  );

  return luceneapi_system_settings_form($form, $module);
}

/**
 * Performance settings for the Search Lucene API module's index.
 *
 * @param &$form_state
 *   A keyed array containing the current state of the form.
 * @param $module
 *   A string containing the Search Lucene API module.
 * @return
 *   A FAPI array passed through system_settings_form().
 */
function luceneapi_admin_settings_performance_form(&$form_state, $module) {
  $form = array();

  // index settings fieldset
  $form['search'] = array(
    '#type' => 'fieldset',
    '#title' => t('Search performance'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );

  // option to optimize the index
  $form['search']['optimize'] = array(
    '#type' => 'submit',
    '#value' => t('Optimize index'),
  );

  // optimize the index after it is updated
  $form['search']["{$module}:optimize_on_update"] = array(
    '#type' => 'radios',
    '#title' => t('Optimize on update'),
    '#default_value' => luceneapi_variable_get($module, 'optimize_on_update'),
    '#options' => array(
      1 => t('Yes'),
      0 => t('No'),
    ),
    '#description' => t(
      'Setting to <em>Yes</em> will automatically optimize the index after it is updated.'
    ),
  );

  $description = 'Helps limit score calculation when searching larger indexes.'
               .' This greatly reduces both search time and memory consumption.'
               .' Note that the first <i>n</i> number of results are returned,'
               .' not the <i>n</i> best number of results.';

  // result set limitation to prevent excessive score calculations
  $options = drupal_map_assoc(array(-1, 500, 1000, 2000, 5000, 10000));
  $options[-1] = 'No limit';
  $form['search']["{$module}:resultset_limit"] = array(
    '#type' => 'select',
    '#title' => t('Result set limit'),
    '#default_value' => luceneapi_variable_get($module, 'resultset_limit', TRUE),
    '#options' => $options,
    '#description' => t($description),
  );

  $description = 'Cache search results and other resource intensive items.'
               .' Setting to <em>Yes</em> greatly improves the overall'
               .' performance of the module.';

  // cache expensive operations
  $form['search']["{$module}:caching_enabled"] = array(
    '#type' => 'radios',
    '#title' => t('Enable caching'),
    '#default_value' => luceneapi_variable_get($module, 'caching_enabled'),
    '#options' => array(
      1 => t('Yes'),
      0 => t('No'),
    ),
    '#description' => t($description),
  );

  // default results per page for all modules that implement SLAPI
  $form['search']["{$module}:number_cached"] = array(
    '#type' => 'select',
    '#title' => t('Maximum size of cached result set'),
    '#default_value' => luceneapi_variable_get($module, 'number_cached'),
    '#options' => drupal_map_assoc(array(100, 200, 500, 1000, 2000, 5000, 10000)),
    '#description' => t(
      'The first <i>n</i> number of results in a result set will be cached.'
    ),
  );

  // option to clear the search results
  $form['search']['clear'] = array(
    '#type' => 'submit',
    '#value' => t('Clear search results cache'),
  );

  // index settings fieldset
  $form['index'] = array(
    '#type' => 'fieldset',
    '#title' => t('Indexing performance'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );

  $description = 'The minimal number of documents required before the buffered'
               .' in-memory documents are written into a new segment.  Setting'
               .' to a higher value may improve indexing performance but will'
               .' also increase memory consumption.';

  // sets largest maximum beffered documents setting
  $form['index']["{$module}:max_buffered_docs"] = array(
    '#type' => 'select',
    '#title' => t('Maximum buffered documents'),
    '#default_value' => luceneapi_variable_get($module, 'max_buffered_docs'),
    '#options' => drupal_map_assoc(array(10, 20, 50, 100, 200, 500, 1000)),
    '#description' => t($description),
  );

  $description = 'Lower values increase the quality of unoptimized indexes,'
               .' while larger values increase indexing performance. Setting'
               .' the value too high may trigger the <em>too many open files'
               .' </em> error determined by OS limitations.';

  // determines how often segment indices are merged by addDocument().
  $form['index']["{$module}:merge_factor"] = array(
    '#type' => 'select',
    '#title' => t('Merge factor'),
    '#default_value' => luceneapi_variable_get($module, 'merge_factor'),
    '#options' => drupal_map_assoc(array(10, 20, 50, 100, 200, 500, 1000)),
    '#description' => t($description),
  );

  return luceneapi_system_settings_form($form, $module);
}

/**
 * Displays statistics for the index.
 *
 * @param &$form_state
 *   A keyed array containing the current state of the form.
 * @param $module
 *   A string containing the Search Lucene API module.
 * @return
 *   A FAPI array.
 */
function luceneapi_admin_settings_statistics_form(&$form_state, $module) {
  $form = array();
  try {
    if (!$index = luceneapi_index_open($module, $errstr)) {
      throw new LuceneAPI_Exception($errstr);
    }

    // adds term count to form
    $form['index'] = array(
      '#type' => 'fieldset',
      '#title' => t('Index overview'),
      '#collapsible' => TRUE
    );

    // calculates the percentage of documents indexed
    if ($status = module_invoke($module, 'search', 'status')) {
      $num_left = $status['total'] - $status['remaining'];
      $percentage = ((int)min(100, 100 * ($num_left) / max(1, $status['total']))) .'%';
      $form['index']['percentage'] = array(
        '#value' => t(
          '<p><strong>%percentage</strong> of the content has been indexed.</p>',
          array('%percentage' => $percentage)
        ),
      );
    }

    // finds the number of terms for each field in the index
    $fields = luceneapi_index_fields_get($index, TRUE);
    $fields = array_fill_keys(array_values($fields), 0);
    $terms = luceneapi_index_terms_get($index, TRUE);
    $term_count = count($terms);
    foreach ($terms as $term) {
      $fields[$term->field]++;
    }

    $form['index']['documents'] = array(
      '#value' => sprintf(
        '<p>%s: <em>%d</em></p>',
        t('Number of documents in index'),
        luceneapi_index_size_get($index, FALSE, TRUE)
      ),
    );
    $form['index']['terms'] = array(
      '#value' => sprintf('<p>%s: <em>%d</em></p>', t('Number of terms'), $term_count),
    );
    $form['index']['fields'] = array(
      '#value' => sprintf('<p>%s: <em>%d</em></p>', t('Number of fields'), count($fields)),
    );

    // builds table
    $headers = array(
      t('Lucene field'),
      t('Number of unique terms')
    );
    $rows = array();
    foreach (array_keys($fields) as $field) {
      $rows[] = array($field, $fields[$field]);
    }
    if (count($rows)) {
      $form['terms']['#value'] = theme('table', $headers, $rows);
    }

  }
  catch (LuceneAPI_Exception $e) {
    luceneapi_throw_error($e, NULL, $module);
  }

  return $form;
}

/**
 * Validate function for admin forms, clears luceneapi cache items.
 *
 * @param $form
 *   A FAPI array modeling the submitted form.
 * @param &$form_state
 *   An array containing the current state of the form.
 * @return
 *   NULL
 */
function luceneapi_admin_settings_form_validate($form, &$form_state) {
  if (!in_array($form_state['values']['module'], luceneapi_module_list())) {
    form_set_error('module', t(
      'Module %module is not a valid Search Lucene API module.',
      array('%module' => $form_state['values']['module'])
    ));
    return;
  }
  if ($form_state['values']['op'] == t('Optimize index')) {
    drupal_goto(sprintf('admin/settings/%s/optimize', $form_state['values']['module']));
  }
  if ($form_state['values']['op'] == t('Clear search results cache')) {
    drupal_goto(sprintf('admin/settings/%s/cache', $form_state['values']['module']));
  }
  if ($form_state['values']['op'] == t('Re-index')) {
    drupal_goto(sprintf('admin/settings/%s/reindex', $form_state['values']['module']));
  }
  if ($form_state['values']['op'] == t('Wipe index')) {
    drupal_goto(sprintf('admin/settings/%s/wipe', $form_state['values']['module']));
  }
}

/**
 * Submit handler, clears Search Lucene API caches.
 *
 * @param $form
 *   A FAPI array modeling the submitted form.
 * @param &$form_state
 *   An array containing the current state of the form.
 * @return string
 *   NULL
 */
function luceneapi_admin_settings_form_submit($form, &$form_state) {
  foreach (luceneapi_module_list() as $module) {
    cache_clear_all($module, 'cache', TRUE);
  }
}

/**
 * Formats the return path.
 *
 * @param $module
 *   A string containing the Search Lucene API module.
 * @param $task
 *   A string containing the task.
 * @return
 *   A string containing the path.
 */
function _luceneapi_return_path_get($module, $task) {
  $path = ('optimize' == $task || 'cache' == $task) ? 'performance' : '';
  return rtrim(sprintf('admin/settings/%s/%s', $module, $path), '/');
}

/**
 * Confirmation form for admin tasks.
 *
 * @param &$form_state
 *   An array containing the current state of the form.
 * @param $module
 *   A string containing the module managing the index being optimized.
 * @param $task
 *   A string containing the task type, i.e. optimize
 * @return
 *   NULL
 * @ingroup forms
 */
function luceneapi_admin_confirm(&$form_state, $module, $task) {
  $return_path = _luceneapi_return_path_get($module, $task);
  switch ($task) {
    case 'optimize':
      $button_text = 'Optimize index';
      $confim_message = 'Are you sure you want to optimize the index?';
      $description = 'A Lucene index consists of many segments. Each segment is'
                   .' a completely independent set of data. New documents are'
                   .' added to the index by creating new segment. Increasing'
                   .' number of segments reduces quality of the index, but'
                   .' index optimization restores it.  This process may take a'
                   .' while for larger indexes, so please be patient.';
      break;

    case 'cache':
      $button_text = 'Clear cache';
      $confim_message = 'Are you sure you want to clear the results cache?';
      $description = 'Maintaing a search results cache can reduce the time it'
                   .' takes to search and sort content, but it can become very'
                   .' large over time.  It may also store results that are'
                   .' out of date or are no longer valid';
      break;

    case 'reindex':
      $button_text = 'Reindex';
      $confim_message = 'Are you sure you want to rebuild the index?';
      $description = 'Reindexing the site will overwrite all entries currently'
                   .' in the index.  Although the index can still be searched,'
                   .' it may take many cron runs to rebuild it completely.';
      break;

    case 'wipe':
      $button_text = 'Wipe index';
      $confim_message = 'Are you sure you want to wipe the index?';
      $description = 'Wiping the index will immediately remove all entries'
                   .' leaving the site search unusable.  It may take many cron'
                   .' runs to rebuild the index.  If the index becomes corrupt,'
                   .' this may be the only option to repair the site search.';
      break;

    default:
      drupal_goto($return_path);
      break;
  }

  // returns the confirmation form
  return confirm_form(
    array(
      'module' => array(
        '#type' => 'hidden',
        '#value' => $module,
      ),
      'task' => array(
        '#type' => 'hidden',
        '#value' => $task,
      ),
    ),
    t($confim_message),
    $return_path,
    t($description),
    t($button_text),
    t('Cancel')
  );
}

/**
 * Validate function for the admin functions that require confirmation such as
 * clearing the cache and index optimization.
 *
 * @param $form
 *   A FAPI array modeling the submitted form.
 * @param &$form_state
 *   An array containing the current state of the form.
 * @return
 *   NULL
 */
function luceneapi_admin_confirm_validate($form, &$form_state) {
  if (!in_array($form_state['values']['module'], luceneapi_module_list())) {
    form_set_error('module', t(
      '%module is not a valid Search Lucene API module',
      array('%module' => $form_state['values']['module'])
    ));
  }
  $valid_tasks = array('optimize', 'cache', 'reindex', 'wipe');
  if (!in_array($form_state['values']['task'], $valid_tasks)) {
    form_set_error('task', t(
      '%task is not a valid task',
      array('%task' => $form_state['values']['task'])
    ));
  }
}

/**
 * Submit handler for the admin functions that require confirmation such as
 * clearing the cache and index optimization.
 *
 * @param $form
 *   A FAPI array modeling the submitted form.
 * @param &$form_state
 *   An array containing the current state of the form.
 * @return string
 *   NULL
 */
function luceneapi_admin_confirm_submit($form, &$form_state) {
  if ($form_state['values']['confirm']) {
    $return_path = _luceneapi_return_path_get(
      $form_state['values']['module'],
      $form_state['values']['task']
    );
    switch ($form_state['values']['task']) {
      case 'optimize':
        if ($index = luceneapi_index_open($form_state['values']['module'], $errstr)) {
          if (luceneapi_index_optimize($index)) {
            drupal_set_message(t('Index sucessfully optimized'));
          }
        }
        else {
          luceneapi_throw_error($errstr, NULL, $form_state['values']['module']);
        }
        break;

      case 'cache':
        $cid = sprintf('%s:', $form_state['values']['module']);
        cache_clear_all($cid, LUCENEAPI_RESULTS_CACHE_TABLE, TRUE);
        drupal_set_message(t('Cached search results cleared'));
        break;

      case 'reindex':
        module_invoke($form_state['values']['module'], 'search', 'reset');
        drupal_set_message(t('The index will be rebuilt.'));
        break;

      case 'wipe':
        if ($index = luceneapi_index_open($form_state['values']['module'], $errstr)) {
          module_invoke($form_state['values']['module'], 'search', 'reset');
          if (luceneapi_index_wipe($index)) {
            luceneapi_index_optimize($index);
            drupal_set_message(t('Index sucessfully wiped'));
          }
        }
        break;
    }
    $form_state['redirect'] = $return_path;
  }
}

/**
 * Creates directory if it does not exist, adds .htaccess file.
 *
 * @param $form_element
 *   The form element containing the name of the directory to check.
 * @return
 *   An array containing the form element.
 * @see system_check_directory()
 */
function luceneapi_after_build($form_element) {
  luceneapi_check_directory($form_element['#value'], FILE_CREATE_DIRECTORY, $form_element['#parents'][0]);
  return $form_element;
}
